@@ -39,7 +39,7 @@ blue_ = "#7570b3"
 bracket_ = function(..., x, xend = x, y, yend = y, color = red_) {
   annotate("segment",
     arrow = arrow(angle = 90, ends = "both", length = unit(3, "points")),
-    color = color, size = 0.75,
+    color = color, linewidth = 0.75,
     x = x, xend = xend, y = y, yend = yend,
     ...
   )
@@ -47,7 +47,7 @@ bracket_ = function(..., x, xend = x, y, yend = y, color = red_) {
 thickness_ = function(x) dnorm(x,4,1) * 0.9 / dnorm(4,4,1)
 refline_ = function(..., x, xend = x, y, yend = y, color = red_, linetype = "solid", alpha = 0.5) {
   annotate("segment",
-    color = color, linetype = linetype, alpha = alpha, size = 0.75,
+    color = color, linetype = linetype, alpha = alpha, linewidth = 0.75,
     x = x, xend = xend, y = y, yend = yend,
     ...
   )
@@ -74,7 +74,7 @@ tibble(dist = dist_normal(4, 1.2)) %>%
   geom_hline(yintercept = 0:1, color = "gray95") +
 
   stat_dotsinterval(
-    aes(interval_size = NULL),
+    aes(linewidth = NULL),
     slab_color = "gray50", 
     .width = 1 - 2*pnorm(-1, sd = 1.2),
     fill = "gray75",
@@ -82,8 +82,8 @@ tibble(dist = dist_normal(4, 1.2)) %>%
     shape = 22,
     slab_shape = 21,
     stroke = 1.5,
-    interval_size = 5,
-    slab_size = 1.5
+    linewidth = 5,
+    slab_linewidth = 1.5
   ) +
   
   # height
@@ -98,7 +98,7 @@ tibble(dist = dist_normal(4, 1.2)) %>%
   
   # slab line properties
   label_(x = 2.5, y = 0.7, 
-    label = 'slab_color = "gray50"\nslab_size = 1.5',
+    label = 'slab_color = "gray50"\nslab_linewidth = 1.5',
     vjust = 1, hjust = 1
   ) +
   arrow_(x = 2.52, xend = 3, y = 0.67, yend = thickness_(3.1) + 0.03, curvature = -0.2) +
@@ -120,7 +120,7 @@ tibble(dist = dist_normal(4, 1.2)) %>%
   
   # interval properties
   label_(x = 3.5, y = -0.2,
-    label = 'interval_color = color = "black"\ninterval_alpha = alpha = 1\ninterval_linetype = linetype = "solid"\ninterval_size = size = 5',
+    label = 'interval_color = color = "black"\ninterval_alpha = alpha = 1\ninterval_linetype = linetype = "solid"\nlinewidth = size = 5',
     vjust = 1, hjust = 1
   ) +
   arrow_(x = 3.3, xend = 3.4, y = -0.18, yend = -0.015, curvature = -0.1) +
@@ -142,10 +142,10 @@ tibble(dist = dist_normal(4, 1.2)) %>%
   geom_hline(yintercept = 0:1, color = "gray95") +
 
   stat_dots(
-    aes(interval_size = NULL),
+    aes(linewidth = NULL),
     color = "gray50",
     fill = "gray75",
-    size = 1.5,
+    linewidth = 1.5,
     shape = 21
   ) +
   
@@ -161,7 +161,7 @@ tibble(dist = dist_normal(4, 1.2)) %>%
   
   # slab line properties
   label_(x = 2.5, y = 0.7, 
-    label = 'color = "gray50"\nsize = 1.5',
+    label = 'color = "gray50"\nlinewidth = 1.5',
     vjust = 1, hjust = 1
   ) +
   arrow_(x = 2.52, xend = 3, y = 0.67, yend = thickness_(3.1) + 0.03, curvature = -0.2) +
@@ -221,29 +221,59 @@ data.frame(x = c(.4, .7, .7, 1, 1, 1)) %>%
   coord_fixed() +
   labs(subtitle = "Layout parameters for dots geoms", x = NULL, y = NULL)
 
-## ----side_layout, fig.width = small_width, fig.height = small_width * 3/2---------------------------------------------
+## ----horizontal_side, fig.width = small_width, fig.height = small_width/2---------------------------------------------
 set.seed(1234)
 x = rnorm(100)
 
-make_plot = function(layout) {
+side_plot = function(...) {
   expand.grid(
     x = x,
     side = c("topright", "both", "bottomleft"),
     stringsAsFactors = FALSE
   ) %>%
-    ggplot(aes(side = side, x = x)) +
-    stat_dotsinterval(layout = layout) +
+    ggplot(aes(side = side, ...)) +
+    geom_dots() +
     facet_grid(~ side, labeller = "label_both") +
+    labs(x = NULL, y = NULL) +
+    theme(panel.border = element_rect(color = "gray75", fill = NA))
+}
+side_plot(x = x) +
+  labs(title = "Horizontal geom_dots() with different values of side") +
+  scale_y_continuous(breaks = NULL)
+
+## ----vertical_side, fig.width = small_width, fig.height = small_width/2-----------------------------------------------
+side_plot(y = x) +
+  labs(title = "Vertical geom_dots() with different values of side") +
+  scale_x_continuous(breaks = NULL)
+
+## ----layout_top, fig.width = small_width, fig.height = small_height---------------------------------------------------
+layout_plot = function(layout, side, ...) {
+  data.frame(
+    x = x
+  ) %>%
+    ggplot(aes(x = x)) +
+    geom_dots(layout = layout, side = side, stackratio = if (layout == "hex") 0.9 else 1) +
     labs(
-      subtitle = paste0("stat_dotsinterval(layout = '", layout, "')"),
+      subtitle = paste0("layout = ", deparse(layout), if (layout == "hex") " with stackratio = 0.9"),
       x = NULL,
       y = NULL
-    )
+    ) +
+    scale_y_continuous(breaks = NULL) +
+    theme(panel.border = element_rect(color = "gray75", fill = NA))
 }
 
-make_plot("bin") /
-  make_plot("weave") /
-  make_plot("swarm")
+(layout_plot("bin", side = "top") +
+  layout_plot("hex", side = "top")) /
+  (layout_plot("weave", side = "top") +
+  layout_plot("swarm", side = "top")) +
+  plot_annotation(title = 'geom_dots() layouts with side = "top"')
+
+## ----layout_both, fig.width = small_width, fig.height = small_height--------------------------------------------------
+(layout_plot("bin", side = "both") +
+  layout_plot("hex", side = "both")) /
+  (layout_plot("weave", side = "both") +
+  layout_plot("swarm", side = "both")) +
+  plot_annotation(title = 'geom_dots() layouts with side = "both"')
 
 ## ----beeswarm_bin-----------------------------------------------------------------------------------------------------
 set.seed(1234)
@@ -255,14 +285,40 @@ abc_df = data.frame(
 
 abc_df %>%
   ggplot(aes(x = abc, y = value)) +
-  stat_dots(side = "both") +
-  ggtitle('stat_dots(side = "both")')
+  geom_dots(side = "both") +
+  ggtitle('geom_dots(side = "both")')
+
+## ----beeswarm_hex-----------------------------------------------------------------------------------------------------
+abc_df %>%
+  ggplot(aes(x = abc, y = value)) +
+  geom_dots(side = "both", layout = "hex", stackratio = 0.92) +
+  ggtitle('geom_dots(side = "both", layout = "hex")')
 
 ## ----beeswarm---------------------------------------------------------------------------------------------------------
 abc_df %>%
   ggplot(aes(x = abc, y = value)) +
-  stat_dots(side = "both", layout = "swarm") +
-  ggtitle('stat_dots(side = "both", layout = "swarm")')
+  geom_dots(side = "both", layout = "swarm") +
+  ggtitle('geom_dots(side = "both", layout = "swarm")')
+
+## ----geom_weave-------------------------------------------------------------------------------------------------------
+set.seed(1234)
+
+swarm_data = data.frame(
+  y = rnorm(300, c(1,4)),
+  g = c("a","b")
+)
+
+swarm_plot = swarm_data %>%
+  ggplot(aes(x = g, y = y)) +
+  geom_swarm(linewidth = 0) +
+  labs(title = "geom_swarm()")
+
+weave_plot = swarm_data %>%
+  ggplot(aes(x = g, y = y)) +
+  geom_weave(linewidth = 0) +
+  labs(title = "geom_weave()")
+
+swarm_plot + weave_plot
 
 ## ----beeswarm_dodge---------------------------------------------------------------------------------------------------
 set.seed(12345)
@@ -275,43 +331,41 @@ abcc_df = data.frame(
 
 abcc_df %>%
   ggplot(aes(y = value, x = abc, fill = hi)) +
-  geom_dots(side = "both", position = "dodge") +
+  geom_weave(position = "dodge", linewidth = 0) +
   scale_color_brewer(palette = "Dark2") +
   ggtitle(
-    'geom_dots(side = "both", position = "dodge")',
+    'geom_weave(position = "dodge")',
     'aes(fill = hi)'
   )
 
 ## ----beeswarm_shape_color---------------------------------------------------------------------------------------------
 abcc_df %>%
   ggplot(aes(y = value, x = abc, shape = abc, color = hi)) +
-  # we'll also increase the `scale` here since we 
-  # have some extra space from the dodging
-  geom_dots(side = "both", position = "dodge", scale = 1.5) +
+  geom_weave(position = "dodge") +
   scale_color_brewer(palette = "Dark2") +
   ggtitle(
-    'geom_dots(side = "both", position = "dodge")',
+    'geom_weave(position = "dodge")',
     'aes(shape = abc, fill = hi)'
   )
 
 ## ----beeswarm_shape_color_together------------------------------------------------------------------------------------
 abcc_df %>%
-  ggplot(aes(y = value, x = abc, shape = abc, color = hi, group = NA)) +
-  geom_dots() +
+  ggplot(aes(y = value, x = abc, fill = hi, group = NA)) +
+  geom_dots(linewidth = 0) +
   scale_color_brewer(palette = "Dark2") +
   ggtitle(
     'geom_dots()',
-    'aes(shape = abc, color = hi, group = NA)'
+    'aes(fill = hi, group = NA)'
   )
 
 ## ----beeswarm_shape_color_together_stacked----------------------------------------------------------------------------
 abcc_df %>%
-  ggplot(aes(y = value, x = abc, shape = abc, color = hi, group = NA, order = hi)) +
-  geom_dots() +
+  ggplot(aes(y = value, x = abc, fill = hi, group = NA, order = hi)) +
+  geom_dots(linewidth = 0) +
   scale_color_brewer(palette = "Dark2") +
   ggtitle(
     'geom_dots()',
-    'aes(shape = abc, color = hi, group = NA, order = hi)'
+    'aes(fill = hi, group = NA, order = hi)'
   )
 
 ## ----beeswarm_shape_color_continuous----------------------------------------------------------------------------------
@@ -324,14 +378,95 @@ abcc_df %>%
     'aes(color = value)'
   )
 
-## ----beeswarm_shape_color_alpha---------------------------------------------------------------------------------------
-abcc_df %>%
-  arrange(hi) %>%
-  ggplot(aes(y = value, x = abc, shape = abc, color = abc, alpha = abs(value))) +
-  geom_dots(position = "dodge") +
-  ggtitle(
-    'geom_dots(side = "both", layout = "swarm")',
-    'aes(color = value, alpha = abs(value))'
+## ----large_sample_min_binwidth, fig.width = small_width, fig.height = small_width/2-----------------------------------
+set.seed(1234)
+x = rnorm(2000)
+
+ggplot() +
+  geom_dots(aes(x), binwidth = unit(c(1.5, Inf), "mm"), overflow = "compress", alpha = 0.5) +
+  labs(
+    title = 'geom_dots()',
+    subtitle = 'binwidth = unit(c(1.5, Inf), "mm"), overflow = "compress")'
+  )
+
+## ----large_sample_smooth, fig.width = small_width, fig.height = small_width/2-----------------------------------------
+ggplot() +
+  geom_dots(aes(x), smooth = "unbounded") +
+  labs(
+    title = 'geom_dots() with 2000 dots',
+    subtitle = 'smooth = "unbounded"',
+    x = NULL
+  ) +
+  scale_y_continuous(breaks = NULL)
+
+## ----large_sample_smooth_hex, fig.width = small_width, fig.height = small_width/2-------------------------------------
+ggplot() +
+  geom_dots(aes(x), smooth = "unbounded", layout = "hex", stackratio = 0.9, side = "both") +
+  labs(
+    title = 'geom_dots() with 2000 dots',
+    subtitle = 'smooth = "unbounded", layout = "hex", stackratio = 0.9, side = "both")',
+    x = NULL
+  ) +
+  scale_y_continuous(breaks = NULL)
+
+## ----smooth_bounded_versus_unbounded----------------------------------------------------------------------------------
+set.seed(1234)
+x = rbeta(2000, 0.5, 0.5)
+
+ggplot(data.frame(x), aes(x)) +
+  geom_dots(aes(y = "bounded"), smooth = smooth_bounded(bounds = c(0, 1))) +
+  geom_dots(aes(y = "unbounded"), smooth = "unbounded") +
+  geom_vline(xintercept = c(0, 1), alpha = 0.25) +
+  scale_x_continuous(breaks = c(0, 0.5, 1)) +
+  labs(
+    title = "geom_dots(smooth = ...) on x ~ Beta(0.5, 0.5)",
+    y = "smooth ="
+  )
+
+## ----discrete_dots_too_small, fig.width = small_width, fig.height = small_height--------------------------------------
+set.seed(1234)
+abcd_df = data.frame(
+  x = sample(c("a", "b", "c", "d"), 1000, replace = TRUE, prob = c(0.27, 0.6, 0.03, 0.005)),
+  g = c("a","b")
+)
+
+abcd_df %>%
+  ggplot(aes(x = x)) +
+  geom_dots() +
+  scale_y_continuous(breaks = NULL) +
+  labs(
+    title = "geom_dots()",
+    subtitle = "on a large discrete sample"
+  )
+
+## ----discrete_dots_rect, fig.width = small_width, fig.height = small_height-------------------------------------------
+abcd_df %>%
+  ggplot(aes(x = x)) +
+  geom_dots(smooth = "discrete") +
+  scale_y_continuous(breaks = NULL) +
+  labs(
+    title = 'geom_dots(smooth = "discrete")',
+    subtitle = "on a large discrete sample"
+  )
+
+## ----discrete_dots_bar, fig.width = small_width, fig.height = small_height--------------------------------------------
+abcd_df %>%
+  ggplot(aes(x = x)) +
+  geom_dots(smooth = "bar", group = NA) +
+  scale_y_continuous(breaks = NULL) +
+  labs(
+    title = 'geom_dots(smooth = "bar", group = NA)',
+    subtitle = "on a large discrete sample"
+  )
+
+## ----discrete_dots_ep, fig.width = small_width, fig.height = small_height---------------------------------------------
+abcd_df %>%
+  ggplot(aes(x = x)) +
+  geom_dots(smooth = smooth_discrete(kernel = "ep"), side = "both") +
+  scale_y_continuous(breaks = NULL) +
+  labs(
+    title = 'geom_dots(smooth = smooth_discrete(kernel = "ep"), side = "both")',
+    subtitle = "on a large discrete sample"
   )
 
 ## ----dotsinterval_dist------------------------------------------------------------------------------------------------
@@ -359,22 +494,22 @@ dist_df %>%
 
 ## ----dotsinterval_dist_1000_level_color-------------------------------------------------------------------------------
 dist_df %>%
-  ggplot(aes(y = dist_name, xdist = dist, slab_color = stat(level))) +
+  ggplot(aes(y = dist_name, xdist = dist, slab_color = after_stat(level))) +
   stat_dotsinterval(quantiles = 1000, point_interval = mode_hdci, layout = "weave") +
   scale_color_manual(values = scales::brewer_pal()(3)[-1], aesthetics = "slab_color") +
   ggtitle(
     "stat_dotsinterval(quantiles = 1000, point_interval = mode_hdci)",
-    "aes(y = dist_name, xdist = dist, slab_color = stat(level))"
+    "aes(y = dist_name, xdist = dist, slab_color = after_stat(level))"
   )
 
 ## ----dotsinterval_dist_color------------------------------------------------------------------------------------------
 dist_df %>%
-  ggplot(aes(y = dist_name, xdist = dist, slab_color = stat(x))) +
+  ggplot(aes(y = dist_name, xdist = dist, slab_color = after_stat(x))) +
   stat_dotsinterval(slab_shape = 19, quantiles = 500) +
   scale_color_distiller(aesthetics = "slab_color", guide = "colorbar2") +
   ggtitle(
     "stat_dotsinterval(slab_shape = 19, quantiles = 500)",
-    'aes(slab_color = stat(x)) +\nscale_color_distiller(aesthetics = "slab_color", guide = "colorbar2")'
+    'aes(slab_color = after_stat(x)) +\nscale_color_distiller(aesthetics = "slab_color", guide = "colorbar2")'
   )
 
 ## ----dist_dots_shape_color, fig.width = med_width, fig.height = large_height------------------------------------------
@@ -387,12 +522,12 @@ ab_df = tibble(
 ab_df %>%
   ggplot(aes(
     y = ab, xdist = dist_normal(mean, sd),
-    fill = stat(x < 6), shape = stat(x < 6)
+    fill = after_stat(x < 6), shape = after_stat(x < 6)
   )) +
   stat_dots(position = "dodge", color = NA) +
   labs(
     title = "stat_dots()",
-    subtitle = "aes(xdist = dist_normal(mean, sd), fill and shape = stat(x < 6))"
+    subtitle = "aes(xdist = dist_normal(mean, sd), fill and shape = after_stat(x < 6))"
   ) +
   geom_vline(xintercept = 6, alpha = 0.25) +
   scale_x_continuous(breaks = 2:10) +
@@ -401,11 +536,11 @@ ab_df %>%
 
 ## ----dist_dots_weave, fig.width = med_width, fig.height = large_height------------------------------------------------
 ab_df %>%
-  ggplot(aes(y = ab, xdist = dist_normal(mean, sd), fill = stat(x < 6))) +
+  ggplot(aes(y = ab, xdist = dist_normal(mean, sd), fill = after_stat(x < 6))) +
   stat_dots(position = "dodge", color = NA, layout = "weave") +
   labs(
     title = 'stat_dots(layout = "weave")',
-    subtitle = "aes(fill = stat(x < 6))"
+    subtitle = "aes(fill = after_stat(x < 6))"
   ) +
   geom_vline(xintercept = 6, alpha = 0.25) +
   scale_x_continuous(breaks = 2:10)
@@ -418,12 +553,12 @@ data.frame(
   value = rnorm(200, c(1, 8, 8, 3), c(1, 1.5, 1.5, 1))
 ) %>%
   ggplot(aes(y = abc, x = value, fill = abc)) +
-  stat_slab(aes(thickness = stat(pdf*n)), scale = 0.7) +
-  stat_dotsinterval(side = "bottom", scale = 0.7, slab_size = NA) +
+  stat_slab(aes(thickness = after_stat(pdf*n)), scale = 0.7) +
+  stat_dotsinterval(side = "bottom", scale = 0.7, slab_linewidth = NA) +
   scale_fill_brewer(palette = "Set2") +
   ggtitle(paste0(
-      'stat_slab(aes(thickness = stat(pdf*n)), scale = 0.7) +\n',
-      'stat_dotsinterval(side = "bottom", scale = 0.7, slab_size = NA)'
+      'stat_slab(aes(thickness = after_stat(pdf*n)), scale = 0.7) +\n',
+      'stat_dotsinterval(side = "bottom", scale = 0.7, slab_linewidth = NA)'
     ),
     'aes(fill = abc)'
   )
@@ -433,11 +568,12 @@ gentoo = penguins %>%
   filter(species == "Gentoo", !is.na(sex))
 
 gentoo %>%
-  ggplot(aes(x = body_mass_g, y = sex, side = ifelse(sex == "male", "bottom", "top"))) +
+  ggplot(aes(x = body_mass_g, y = sex, side = sex)) +
   geom_dots(scale = 0.5) +
+  scale_side_mirrored(guide = "none") +
   ggtitle(
     "geom_dots(scale = 0.5)",
-    'aes(side = ifelse(sex == "male", "bottom", "top"))'
+    'aes(side = sex) + scale_side_mirrored()'
   ) 
 
 ## ----m_gentoo---------------------------------------------------------------------------------------------------------
@@ -461,16 +597,18 @@ prediction_grid %>%
   ) %>%
   ggplot(aes(x = body_mass_g)) +
   geom_dots(
-    aes(y = as.numeric(sex == "male"), side = ifelse(sex == "male", "bottom", "top")),
+    aes(y = as.numeric(sex == "male"), side = sex),
     scale = 0.4,
     data = gentoo
   ) +
   stat_lineribbon(
     aes(ydist = p_male), alpha = 1/4, fill = "#08306b"
   ) +
+  scale_side_mirrored(guide = "none") +
+  coord_cartesian(ylim = c(0, 1)) +
   labs(
     title = "logit dotplot: stat_dots() with stat_lineribbon()",
-    subtitle = 'aes(side = ifelse(sex == "male", "bottom", "top"))',
+    subtitle = 'aes(side = sex) + scale_side_mirrored()',
     x = "Body mass (g) of Gentoo penguins",
     y = "Pr(sex = male)"
   )
