@@ -57,24 +57,45 @@ test_that("binning works on symmetric distributions", {
   expect_equal(automatic_bin(c(1,2), width = 1),
     list(bins = c(1, 2), bin_midpoints = c(1, 2))
   )
+
+  expect_equal(automatic_bin(c(1:5, 5.5), width = 4),
+    list(bins = c(1, 1, 1, 2, 2, 2), bin_midpoints = c(2, 4.75))
+  )
+
+  expect_equal(automatic_bin(1:8, width = 2.1),
+    list(bins = c(1, 1, 2, 2, 3, 3, 4, 4), bin_midpoints = c(1.5, 3.5, 5.5, 7.5))
+  )
+})
+
+test_that("bin sweeping fixes edge effects", {
+  x = sort(round(-qexp(ppoints(20), 1/4)))
+  automatic_bin(x, 2)
+
+  expect_equal(
+    automatic_bin(x, 2),
+    list(
+      bins = c(1,2,3,3,4,4,5,5,5,5,5,5,5,5,6,6,6,6,6,6),
+      bin_midpoints = c(-15,-10,-7.5,-5.5,-3,-0.5)
+    )
+  )
 })
 
 test_that("binning works on empty data", {
 
   expect_equal(automatic_bin(NULL, width = 1),
-    list(bins = integer(0), bin_midpoints = NULL)
+    list(bins = integer(0), bin_midpoints = numeric(0))
   )
 
   expect_equal(wilkinson_bin_from_center(NULL, width = 1),
-    list(bins = integer(0), bin_midpoints = NULL)
+    list(bins = integer(0), bin_midpoints = numeric(0))
   )
 
   expect_equal(wilkinson_bin_to_right(NULL, width = 1),
-    list(bins = integer(0), bin_midpoints = NULL)
+    list(bins = integer(0), bin_midpoints = numeric(0), bin_left = numeric(0), bin_right = numeric(0))
   )
 
-  expect_equal(wilkinson_bin_to_left(NULL, width = 1),
-    list(bins = integer(0), bin_midpoints = NULL)
+  expect_equal(wilkinson_bin(NULL, width = 1),
+    list(bins = integer(0), bin_midpoints = numeric(0))
   )
 })
 
@@ -130,16 +151,16 @@ test_that("bin layouts work", {
   expect_equal(as.data.frame(bin_dots(1:5, 0, binwidth = 2, layout = "bin")), ref)
 
   ref = data.frame(
-    x = c(1, 2, 3, 4, 5),
-    y = c(1, 3, 1, 3, 1),
-    bin = c(1L,  1L, 2L, 2L, 3L)
+    x = c(1, 2, 3, 4.5, 5.1),
+    y = c(0.75, 2.25, 0.75, 0.75, 2.25),
+    bin = c(1L,  1L, 2L, 3L, 3L)
   )
-  expect_equal(as.data.frame(bin_dots(1:5, 0, binwidth = 2, layout = "weave")), ref)
+  expect_equal(as.data.frame(bin_dots(c(1:3, 4.5, 5.1), 0, binwidth = 1.5, layout = "weave")), ref)
 
   ref = data.frame(
     x = 1:5,
     y = c(1, 2.73205080756888, 1, 2.73205080756888,  1),
-    bin = c(1, 1, 2, 2, 3)
+    bin = c(1, 1, 2, 3, 3)
   )
   expect_equal(bin_dots(1:5, 0, binwidth = 2, layout = "swarm"), ref)
 
@@ -152,3 +173,8 @@ test_that("find_dotplot_binwidth edge cases work", {
   # are next to each other the max is chosen correctly
   expect_equal(find_dotplot_binwidth(c(1,1,2,2,3,3,4,4), 1), 0.5)
 })
+
+
+test_that("small bins work", {
+  expect_equal(wilkinson_bin(c(0, .Machine$double.eps*2), .Machine$double.eps)$bins, 1:2)
+})
